# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import builtins

def main() -> None:
    r"""
    CLI main function.

    ## Details

    It should be called after [`init`].

    ---

    [`init`]: https://docs.rs/poodio/latest/poodio/cli/fn.init.html
    """
    ...

def version() -> builtins.str:
    r"""
    The version tag for [`poodio`].

    ---

    [`poodio`]: https://docs.rs/poodio
    """
    ...
